/** \file
	\brief Interface description for plugins
	\author Tomasz Ostrowski, http://tomeko.net
	\date April 2011

    This file should be shared by main application and device dll. You MUST
    declare _EXPORTING macro before including this file in dll sources.
*/

#ifndef PluginInterfaceH
#define PluginInterfaceH
//---------------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif

#include <windows.h>

/** \def DECLARE_FN
    \brief Declare function or consistent function + function pointer set depending on _EXPORTING
*/
#ifdef _EXPORTING
#define DECLARE_FN(type, fn, ...) __declspec(dllexport) type __stdcall fn(__VA_ARGS__)
#else
#define DECLARE_FN(type, fn, ...) __declspec(dllimport) type __stdcall fn(__VA_ARGS__); \
	typedef type __stdcall (*pf##fn)(__VA_ARGS__)
#endif

/** \brief Dll interface version, used to dismiss outdated libraries
*/
enum { DLL_INTERFACE_MAJOR_VERSION = 1 };
/** \brief Dll interface version, used to dismiss outdated libraries
*/
enum { DLL_INTERFACE_MINOR_VERSION = 0 };

/** \brief Holds Dll interface version
*/
struct S_DLL_INTERFACE
{
	int majorVersion;   ///< major part of interface version
	int minorVersion;   ///< minor part of interface version
};

/** \brief State of connection
*/
enum E_CONNECTION_STATE
{
	OTERM_DISCONNECTED = 0,
	OTERM_CONNECTED
};

///////////////////////////////////////////////////////////////////////////////
// CALLBACKS
///////////////////////////////////////////////////////////////////////////////

typedef void __stdcall (*CALLBACK_SEND)(void *cookie, void *buffer, unsigned int buffersize);


///////////////////////////////////////////////////////////////////////////////
// EXPORTED/IMPORTED FUNCTION SET
///////////////////////////////////////////////////////////////////////////////

/** \brief Get dll interface version to check for compatibility
*/
DECLARE_FN(void, GetInterfaceDescription, struct S_DLL_INTERFACE*);

/** \brief Set dll callbacks (to i.e. send new data from dll to main application)
*/
DECLARE_FN(void, SetCallbacks, void *cookie, CALLBACK_SEND lpSend);

DECLARE_FN(void, ChangeConnectionState, int state);

DECLARE_FN(void, OnRx, void *buffer, unsigned int buffersize);

DECLARE_FN(void, OnTx, void *buffer, unsigned int buffersize);

/** \brief Show dll specific settings window (if available)
*/
DECLARE_FN(void, ShowSettings, HANDLE parent);

#ifdef __cplusplus
}
#endif

#endif
